@@ -18,6 +18,7 @@
 #include "string.hpp"
 #include "cache-t.hpp"
 #include "getdata.hpp"
+#include "file_util.hpp"
 
 #ifdef ENABLE_NLS
 #  include <langinfo.h>
@@ -68,6 +69,9 @@ namespace aspeller {
     , {"sug-split-chars",     KeyInfoString, "- ", "", 0, FOR_CONFIG}
     , {"store-as",            KeyInfoString, "", ""}
     , {"try",                 KeyInfoString, "", ""}
+    , {"normalize",           KeyInfoBool, "false", "", 0, FOR_CONFIG}
+    , {"norm-required",       KeyInfoBool, "false", "", }
+    , {"norm-form",           KeyInfoString, "nfc", "", 0, FOR_CONFIG}
   };
 
   static GlobalCache<Language> language_cache("language");
@@ -112,21 +116,23 @@ namespace aspeller {
 #ifdef ENABLE_NLS
       const char * tmp = 0;
       tmp = bind_textdomain_codeset("aspell", 0);
+#ifdef HAVE_LANGINFO_CODESET
       if (!tmp) tmp = nl_langinfo(CODESET);
+#endif
       if (is_ascii_enc(tmp)) tmp = 0;
       if (tmp)
-        RET_ON_ERR(mesg_conv_.setup(*config, charset_, fix_encoding_str(tmp, buf)));
+        RET_ON_ERR(mesg_conv_.setup(*config, charset_, fix_encoding_str(tmp, buf), NormTo));
       else 
 #endif
-        RET_ON_ERR(mesg_conv_.setup(*config, charset_, data_encoding_));
+        RET_ON_ERR(mesg_conv_.setup(*config, charset_, data_encoding_, NormTo));
       // no need to check for errors here since we know charset_ is a
       // supported encoding
-      to_utf8_.setup(*config, charset_, "utf-8");
-      from_utf8_.setup(*config, "utf-8", charset_);
+      to_utf8_.setup(*config, charset_, "utf-8", NormTo);
+      from_utf8_.setup(*config, "utf-8", charset_, NormFrom);
     }
     
     Conv iconv;
-    RET_ON_ERR(iconv.setup(*config, data_encoding_, charset_));
+    RET_ON_ERR(iconv.setup(*config, data_encoding_, charset_, NormFrom));
 
     DataPair d;
 
@@ -265,8 +271,14 @@ namespace aspeller {
       find_file(repl_file, dir1, dir2, repl, "_repl", ".dat");
       RET_ON_ERR(REPL.open(repl_file, "r"));
       
-      while (getdata_pair(REPL, d, buf), ::to_lower(d.key), d.key != "rep");
-      size_t num_repl = atoi(d.value); // FIXME make this more robust
+      size_t num_repl = 0;
+      while (getdata_pair(REPL, d, buf)) {
+        ::to_lower(d.key);
+        if (d.key == "rep") {
+          num_repl = atoi(d.value); // FIXME make this more robust
+          break;
+        }
+      }
       repls_.resize(num_repl);
 
       for (size_t i = 0; i != num_repl; ++i) {
@@ -285,10 +297,11 @@ namespace aspeller {
 
   void Language::set_lang_defaults(Config & config)
   {
+    config.replace_internal("actual-lang", name());
     StackPtr<KeyInfoEnumeration> els(lang_config_->possible_elements(false));
     const KeyInfo * k;
     Conv to_utf8;
-    to_utf8.setup(config, data_encoding_, "utf-8");
+    to_utf8.setup(config, data_encoding_, "utf-8", NormTo);
     while ((k = els->next()) != 0) {
       if (k->other_data == FOR_CONFIG 
 	  && lang_config_->have(k->name) && !config.have(k->name))
@@ -487,7 +500,7 @@ namespace aspeller {
 	if (!l.special(*i).middle) {
           char m[70];
           snprintf(m, 70, 
-                   _("The character '%s' may not appear at the middle of a word."),
+                   _("The character '%s' may not appear in the middle of a word."),
                    MsgConv(l)(*i));
           return make_err(invalid_word, MsgConv(l)(word), m);
         }
@@ -542,15 +555,14 @@ namespace aspeller {
   PosibErr<Language *> new_language(const Config & config, ParmString lang)
   {
     if (!lang)
-      return get_cache_data(&language_cache, &config, config.retrieve("actual-lang"));
+      return get_cache_data(&language_cache, &config, config.retrieve("lang"));
     else
       return get_cache_data(&language_cache, &config, lang);
   }
 
   PosibErr<void> open_affix_file(const Config & c, FStream & f)
   {
-
-    String lang = c.retrieve("actual-lang");
+    String lang = c.retrieve("lang");
 
     String dir1,dir2,path;
     fill_data_dir(&c, dir1, dir2);
@@ -567,4 +579,27 @@ namespace aspeller {
     return no_err;
   }
 
+  bool find_language(Config & c)
+  {
+    String lang = c.retrieve("lang");
+    lang.ensure_null_end();
+    char * l = lang.data();
+
+    String dir1,dir2,path;
+    fill_data_dir(&c, dir1, dir2);
+
+    char * s = l + strlen(l);
+
+    while (s > l) {
+      find_file(path,dir1,dir2,l,".dat");
+      if (file_exists(path)) {
+        c.replace_internal("actual-lang", lang);
+        return true;
+      }
+      while (s > l && !(*s == '-' || *s == '_')) --s;
+      *s = '\0';
+    }
+    return false;
+  }
+
 }
