@@ -496,10 +496,10 @@ namespace aspeller {
     }
 
     PosibErr<Convert *> conv;
-    conv = new_convert(*c, user_enc, sys_enc);
+    conv = new_convert(*c, user_enc, sys_enc, NormFrom);
     if (conv.has_err()) return conv;
     to_internal_.reset(conv);
-    conv = new_convert(*c, sys_enc, user_enc);
+    conv = new_convert(*c, sys_enc, user_enc, NormTo);
     if (conv.has_err()) return conv;
     from_internal_.reset(conv);
 
