@@ -165,8 +165,8 @@ PosibErr<void> WritableBase::set_file_encoding(ParmString enc, const Config * c)
 {
   if (enc == file_encoding) return no_err;
   if (enc == "") enc = lang()->charset();
-  RET_ON_ERR(iconv.setup(*c, enc, lang()->charset()));
-  RET_ON_ERR(oconv.setup(*c, lang()->charset(), enc));
+  RET_ON_ERR(iconv.setup(*c, enc, lang()->charset(), NormTo));
+  RET_ON_ERR(oconv.setup(*c, lang()->charset(), enc, NormFrom));
   if (iconv || oconv) 
     file_encoding = enc;
   else
