@@ -337,7 +337,7 @@ int main (int argc, const char *argv[])
   if (!utf8_opts.empty()) {
     Conv to_utf8;
 #ifdef USE_LOCALE
-    EXIT_ON_ERR(to_utf8.setup(*options, codeset, "utf-8"));
+    EXIT_ON_ERR(to_utf8.setup(*options, codeset, "utf-8", NormTo));
 #endif
     for (Vector<StringPair>::iterator i = utf8_opts.begin(); 
          i != utf8_opts.end();
@@ -439,7 +439,8 @@ static Convert * setup_conv(const aspeller::Language * lang,
   if (config->retrieve("encoding") != "none") {
     PosibErr<Convert *> pe = new_convert_if_needed(*config,
                                                    lang->charset(),
-                                                   config->retrieve("encoding"));
+                                                   config->retrieve("encoding"),
+                                                   NormTo);
     if (pe.has_err()) {print_error(pe.get_err()->mesg); exit(1);}
     return pe.data;
   } else {
@@ -453,7 +454,8 @@ static Convert * setup_conv(Config * config,
   if (config->retrieve("encoding") != "none") {
     PosibErr<Convert *> pe = new_convert_if_needed(*config,
                                                    config->retrieve("encoding"),
-                                                   lang->charset());
+                                                   lang->charset(),
+                                                   NormFrom);
     if (pe.has_err()) {print_error(pe.get_err()->mesg); exit(1);}
     return pe.data;
   } else {
@@ -1166,7 +1168,7 @@ void list()
 void convt()
 {
   Conv conv;
-  conv.setup(*options, args[0], args[1]);
+  conv.setup(*options, args[0], args[1], NormNone);
   String line;
   while (CIN.getline(line))
     COUT.printl(conv(line));
@@ -1266,6 +1268,7 @@ void master () {
 
   if (action == do_create) {
     
+    find_language(*config);
     EXIT_ON_ERR(create_default_readonly_dict
                 (new IstreamVirEnumeration(CIN),
                  *config));
@@ -1414,6 +1417,7 @@ void repl() {
 void soundslike() {
   using namespace aspeller;
   CachePtr<Language> lang;
+  find_language(*options);
   PosibErr<Language *> res = new_language(*options);
   if (!res) {print_error(res.get_err()->mesg); exit(1);}
   lang.reset(res.data);
@@ -1437,6 +1441,7 @@ void munch()
 {
   using namespace aspeller;
   CachePtr<Language> lang;
+  find_language(*options);
   PosibErr<Language *> res = new_language(*options);
   if (!res) {print_error(res.get_err()->mesg); exit(1);}
   lang.reset(res.data);
@@ -1474,6 +1479,7 @@ void expand()
   
   using namespace aspeller;
   CachePtr<Language> lang;
+  find_language(*options);
   PosibErr<Language *> res = new_language(*options);
   if (!res) {print_error(res.get_err()->mesg); exit(1);}
   lang.reset(res.data);
@@ -1559,6 +1565,7 @@ void combine()
 {
   using namespace aspeller;
   CachePtr<Language> lang;
+  find_language(*options);
   PosibErr<Language *> res = new_language(*options);
   if (!res) {print_error(res.get_err()->mesg); exit(1);}
   lang.reset(res.data);
